<?php
defined( 'ABSPATH' ) or die();

if ( ! class_exists( 'WC_ZenPay_PG' ) ) {
	class WC_ZenPay_PG extends WC_Payment_Gateway {
		var $id                 = null;
		var $icon               = null;
		var $description        = null;
		var $order_button_text  = null;
		var $has_fields         = null;
		var $title              = null;
		var $method_title       = null;
		var $method_description = null;
		var $base_url_staging   = null;
		var $base_url           = null;
		var $biller_code        = null;
		var $hash_key           = null;

		public function __construct() {
			// Properties
			$this->id                 = 'wc_zenpay_pg';
			$this->icon               = WC_ZENPAY_PG_LOGO_URL;
			$this->description        = __( 'Pay with ZenPay', 'wc-zenpay-pg' );
			$this->order_button_text  = __( 'Proceed', 'wc-zenpay-pg' );
			$this->has_fields         = true;
			$this->title              = 'ZenPay';
			$this->method_title       = 'ZenPay';
			$this->method_description = __( 'ZenPay Payment Gateway', 'wc-zenpay-pg' );
			$this->base_url_staging   = 'https://paylanes-qa.sprocket.solutions/merchant/web-payment/auth';
			$this->base_url           = 'https://paylanes.sprocket.solutions/merchant/web-payment/auth';

			// Custom Title
			if ( ! empty( $this->get_option( 'title' ) ) ) {
				$this->title = $this->get_option( 'title' );
			}		

			// Methods
			$this->init_form_fields();
			$this->init_settings();

			$this->biller_code = $this->get_option( 'biller_code' );
			$this->hash_key = $this->get_option( 'hash_key' );

			if ( $this->get_option( 'staging' ) == 'yes' ) {
				$this->base_url = $this->base_url_staging;
			}

			// Hooks
			add_action( 'woocommerce_update_options_payment_gateways_' . $this->id, array( $this, 'process_admin_options' ) );
		}

		public function init_form_fields() {
			$this->form_fields = array(
				'enabled' => array(
					'title'       => __( 'Enable/Disable', 'wc-zenpay-pg' ),
					'type'        => 'checkbox',
					'label'       => __( 'Enable ZenPay Payment Method', 'wc-zenpay-pg' ),
					'default'     => 'yes'
				),
				'title' => array(
					'title'       => __( 'Title', 'wc-zenpay-pg' ),
					'type'        => 'text',
					'description' => __( 'This controls the title which the user sees during checkout.', 'wc-zenpay-pg' ),
					'default'     => __( 'ZenPay', 'wc-zenpay-pg' ),
					'desc_tip'    => true
				),
				'description' => array(
					'title'       => __( 'Description', 'wc-zenpay-pg' ),
					'type'        => 'text',
					'description' => __( 'This controls the description which the user sees during checkout.', 'wc-zenpay-pg' ),
					'default'     => __( 'Pay with ZenPay', 'wc-zenpay-pg' ),
					'desc_tip'    => true
				),
				'biller_code' => array(
					'title'       => __( 'Biller code', 'wc-zenpay-pg' ),
					'type'        => 'text',
					'description' => __( 'Biller code provided by ZenPay', 'wc-zenpay-pg' ),
					'default'     => '',
					'desc_tip'    => true
				),
				'hash_key' => array(
					'title'       => __( 'Hash Key', 'wc-zenpay-pg' ),
					'type'        => 'text',
					'description' => __( 'Hash Key provided by ZenPay', 'wc-zenpay-pg' ),
					'default'     => '',
					'desc_tip'    => true
				),
				'staging' => array(
					'title'       => __( 'Staging mode (sandbox)', 'wc-zenpay-pg' ),
					'type'        => 'checkbox',
					'description' => __( 'Whether to enable staging mode', 'wc-zenpay-pg' ),
					'default'     => 'no',
					'desc_tip'    => true
				)
			);
		}

		public function payment_fields() {
			if ( ! empty( $this->get_option( 'description' ) ) ) {
				echo sprintf( '%s%s%s', '<p>', esc_html( $this->get_option( 'description' ) ), '</p>' );
			}
		}

		public function process_payment( $order_id ) {
			$order = wc_get_order( $order_id );

			if ( empty( $this->biller_code ) || empty( $this->hash_key ) ) {
				wc_add_notice( __( 'Error: ZenPay is not correctly set up.', 'wc-zenpay-pg' ), 'error' );
				return;
			}

			ob_start();
			require WC_ZENPAY_PG_PLUGIN_PATH . 'form.php';
			$form = ob_get_clean();

			return array(
				'result' => 'success',
				'redirect' => '#zenpay_pg',
				'form' => $form
			);
		}
	}
}