<?php
/*
Plugin Name: WooCommerce ZenPay Payment Gateway
Description: Enables ZenPay Payment Gateway for WooCommerce
Version:     1.1.0
Author:      torricelli
Author URI:  https://www.fiverr.com/torricelli
Text Domain: wc-zenpay-pg
Domain Path: /languages
*/

defined( 'ABSPATH' ) or die;

define( 'WC_ZENPAY_PG_PLUGIN_FILE', __FILE__ );
define( 'WC_ZENPAY_PG_PLUGIN_PATH', plugin_dir_path( WC_ZENPAY_PG_PLUGIN_FILE ) );
define( 'WC_ZENPAY_PG_CLASS_PATH', trailingslashit( WC_ZENPAY_PG_PLUGIN_PATH . 'class' ) );
define( 'WC_ZENPAY_PG_INC_PATH', trailingslashit( WC_ZENPAY_PG_PLUGIN_PATH . 'inc' ) );
define( 'WC_ZENPAY_PG_LOGO_URL', plugins_url( 'images/logo.png', WC_ZENPAY_PG_PLUGIN_FILE ) );
define( 'WC_ZENPAY_PG_VER', '1.1.0' );

if ( ! class_exists( 'WC_ZenPay_PG_Wrapper' ) ) {
    class WC_ZenPay_PG_Wrapper {
		public static function get_instance() {
			if ( self::$instance == null ) {
				self::$instance = new self();
			}
			return self::$instance;
        }

		private $inc_dir = null;

		private static $instance = null;

		private function __clone() { }

		public function __wakeup() { }

		private function __construct() {
            // Properties

            // Actions
			add_action( 'init', array( $this, 'load_textdomain' ) );
			add_action( 'plugins_loaded', array( $this, 'check_for_woocommerce' ) );
			add_action( 'rest_api_init', array( $this, 'add_endpoint_url' ) );
			add_action( 'wp_enqueue_scripts', array( $this, 'enqueue_assets' ) );
			add_action( 'woocommerce_thankyou', array( $this, 'woocommerce_thankyou' ) );
        }

		public function load_textdomain() {
			load_plugin_textdomain( 'wc-zenpay-pg', false, dirname( plugin_basename( __FILE__ ) ) . '/languages' );
		}

        public function check_for_woocommerce() {
            if ( ! class_exists( 'WooCommerce' ) ) {
                add_action( 'admin_notices', array( $this, 'missing_wc_notice' ) );
                return;
            }

            define( 'WC_ZENPAY_PG_BASE_FILE', __FILE__ );

            require_once( WC_ZENPAY_PG_CLASS_PATH . 'class-wc-zenpay-pg.php' );

            add_filter( 'woocommerce_payment_gateways', array( $this, 'load_gateway_class' ) );
        }

        public function missing_wc_notice() {
            echo '<div class="error"><p><strong>' . sprintf( esc_html__( 'WooCommerce ZenPay Payment Gateway requires WooCommerce to be installed and active. You can download %s here.', 'wc-zenpay-pg' ), '<a href="https://woocommerce.com/" target="_blank">WooCommerce</a>' ) . '</strong></p></div>';
        }

        public function load_gateway_class( $methods ) {
            $methods[] = 'WC_ZenPay_PG';
            return $methods;
        }

		public function add_endpoint_url() {
			register_rest_route( 'wc-zenpay-pg/v1', '/process/(?P<id>\d+)', array(
				'methods' => array( 'GET', 'POST' ),
				'callback' => array( $this, 'process' ),
				'permission_callback' => '__return_true'
			) );
		}

		public function process( $request ) {
			$order_id = ( int ) $request->get_param( 'id' );
			if ( $order_id < 1 ) {
				die( '-1' );
			}

			$order = wc_get_order( $order_id );
			if ( ! $order ) {
				die( '-2' );
			}

			$st_code = $request->get_param( 'rst_stcode' );
			if ( $st_code == '00' ) {
				$order->update_status( 'completed' );
				$order->add_order_note( sprintf( __( 'Callback processed. Payment completed. st_code is: %1$s, fxp_id is: %2$s', 'wc-zenpay-pg' ), $st_code, $request->get_param( 'rsp_fpx_id' ) ) );
			} else {
				$order->update_status( 'failed' );
				$order->add_order_note( sprintf( __( 'Callback processed. Payment failed. st_code is: %1$s, fxp_id is: %2$s', 'wc-zenpay-pg' ), $st_code, $request->get_param( 'rsp_fpx_id' ) ) );
			}

			$order->save();
			die( '1' );
		}

		public function enqueue_assets() {
			wp_enqueue_script( 'wc-zenpay-pg', plugins_url( 'js/handler.js', WC_ZENPAY_PG_PLUGIN_FILE ), array( 'jquery' ), WC_ZENPAY_PG_VER, true );
			wp_enqueue_style( 'wc-zenpay-pg', plugins_url( 'css/style.css', WC_ZENPAY_PG_PLUGIN_FILE ), array(), WC_ZENPAY_PG_VER, 'all' );
		}

		public static function woocommerce_thankyou( $order_id ) {
			$order = wc_get_order( $order_id );

			if ( $order->get_payment_method() != 'wc_zenpay_pg' ) {
				return;
			}

			if ( $order->get_status() == 'completed' ) {
				return;
			} 

			if ( ! isset( $_GET['ORDER_NUMBER'] ) || intval( $_GET['ORDER_NUMBER'] ) != $order_id ) {
				return;
			}

			// TODO: There must be an API method to query the order status, otherwise a hacky user might mark this order as complete
			// Mandatory: invoke the above method here and validate order status

			// We're safe to proceed at this point
			$payment_id = isset( $_GET['PAYMENT_ID'] ) ? sanitize_text_field( $_GET['PAYMENT_ID'] ) : 'N/A';
			update_post_meta( $order_id, '_wccng_payment_id', $payment_id );

			if ( isset( $_GET['STATUS'] ) ) {
				if ( $_GET['STATUS'] == 'PAID' ) {
					$order->update_status( 'processing', sprintf( __( 'Order paid. CNG Payment ID is %s', 'wc-zenpay-pg' ), $payment_id ) );
				} else {
					$order->update_status( 'failed', sprintf( __( 'Payment failed. CNG returned status %s', 'wc-zenpay-pg' ), $_GET['STATUS'] ) );
				}
			}
		}
    }

	WC_ZenPay_PG_Wrapper::get_instance();
}